import { Component } from '@angular/core';
import { ColDef, CsvExportParams, GridApi, GridReadyEvent, ProcessCellForExportParams } from 'ag-grid-community';
import { Employer } from '../../../models/employer.model';
import { Job } from '../../../models/job.model';
import { Structure } from '../../../models/structure.model';
import { TempUser } from '../../../models/temp-user.model';
import { User } from '../../../models/user.model';
import { NotificationService } from '../../../services/notification.service';
import { TempUserService } from '../../../services/temp-user.service';
import { ButtonTypeV3 } from '../../../shared/components/v3/button/button-type.enum';
import { AdminService } from '../../services/admin.service';
import { AdministredStructuresComponent } from './administred-structures/administred-structures.component';
import { DeleteUserComponent } from './delete-user/delete-user.component';
import { EmployerRendererComponent } from './employer-renderer/employer-renderer.component';
import { JobRendererComponent } from './job-renderer/job-renderer.component';

type UsersStatus = 'attached' | 'unattached';

@Component({
  selector: 'app-admin-manage-users',
  templateUrl: './manage-users.component.html',
  styleUrls: ['../../admin.scss'],
})
export class ManageUsersComponent {
  public attachedUsers: User[] = [];
  public gridApiAttached: GridApi;
  public unAttachedUsers: User[] = [];
  public gridApiUnattached: GridApi;
  public unVerifiedUsers: User[] = [];
  public validatedJobsName: string[] = [];
  public validatedJobs: Job[] = [];
  public validatedEmployersName: string[] = [];
  public validatedEmployers: Employer[] = [];
  public deleteModalOpened = false;
  public editJobModalOpened = false;
  public editEmployerModalOpened = false;
  public userToDelete: User = null;

  public tempUsers: TempUser[];

  public changingJobs: Job[] = [
    { name: null, hasPersonalOffer: true, validated: false },
    { name: null, hasPersonalOffer: true, validated: false },
  ];
  public changingEmployers: Employer[] = [
    { name: null, validated: false },
    { name: null, validated: false },
  ];
  public contextRow: any;
  public buttonTypeEnumV3 = ButtonTypeV3;
  public frameworkComponents;
  public defaultColDef: ColDef = {
    editable: true,
    sortable: true,
    flex: 1,
    minWidth: 100,
    filter: true,
    resizable: true,
  };
  public unAttachedColDef: ColDef = {
    ...this.defaultColDef,
    cellStyle: {
      color: '#da6c2e',
    },
  };

  public columnDefsUnattached: ColDef<User>[];
  public columnDefs: ColDef<User>[] = [
    {
      headerName: 'Nom',
      field: 'surname',
    },
    {
      headerName: 'Prénom',
      field: 'name',
    },
    {
      minWidth: 150,
      headerName: 'Mail',
      field: 'email',
    },
    {
      minWidth: 100,
      headerName: 'Téléphone',
      field: 'phone',
    },
    {
      headerName: 'Structures administrées',
      cellRenderer: 'administredStructuresComponent',
      cellRendererParams: {
        structures: 'structures',
      },
      minWidth: 350,
      field: 'structures',
    },
    {
      headerName: 'Fonction',
      field: 'job',
      cellRenderer: 'jobRenderer',
      cellEditor: 'agSelectCellEditor',
      // Ag select cell editor doesn't accept an object a parameter.
      // Only giving name to display in the select, then recompose the full object when filter on change.
      cellEditorParams: {
        values: this.validatedJobsName,
      },
      onCellValueChanged: this.onJobChange.bind(this),
    },
    {
      headerName: 'Employeur',
      field: 'employer',
      cellRenderer: 'employerRenderer',
      cellEditor: 'agSelectCellEditor',
      cellEditorParams: {
        values: this.validatedEmployersName,
      },
      onCellValueChanged: this.onEmployerChange.bind(this),
    },
    {
      headerName: 'Date de création',
      field: 'createdAt',
      minWidth: 150,
    },
    {
      headerName: 'Actions',
      editable: false,
      minWidth: 150,
      cellRenderer: 'deleteUserComponent',
      cellRendererParams: {
        onClick: this.onDeleteButtonClick.bind(this),
        label: 'Supprimer',
      },
      cellStyle: { 'text-align': 'center' },
    },
  ];

  public colDefsTempUsers: ColDef<TempUser>[] = [
    {
      minWidth: 100,
      headerName: 'E-mail',
      field: 'email',
    },
    {
      minWidth: 200,
      headerName: 'Structure(s)',
      field: 'structures',
      valueGetter: this.getStructureNames.bind(this),
      cellRenderer: 'agGroupCellRenderer',
      autoHeight: true,
    },
    {
      minWidth: 50,
      headerName: 'Demande de rattachement envoyée le',
      field: 'updatedAt',
      valueGetter(params): string {
        return new Date(params.data?.updatedAt).toLocaleDateString();
      },
    },
  ];

  constructor(
    private adminService: AdminService,
    private tempUserService: TempUserService,
    private notificationService: NotificationService
  ) {
    this.findValidatedJobs();
    this.findValidatedEmployers();
    this.columnDefsUnattached = [...this.columnDefs];
    this.columnDefsUnattached = this.columnDefsUnattached.filter((item) =>
      String(item.headerName).toLowerCase().indexOf('structure')
    );
    this.columnDefsUnattached.splice(this.columnDefsUnattached.length - 1, 0, {
      headerName: 'Non rattaché depuis',
      field: 'unattachedSince',
      minWidth: 180,
    });
    this.frameworkComponents = {
      deleteUserComponent: DeleteUserComponent,
      jobRenderer: JobRendererComponent,
      employerRenderer: EmployerRendererComponent,
      administredStructuresComponent: AdministredStructuresComponent,
    };
    this.findAttachedUsers();
    this.findUnAttachedUsers();
    this.findUnVerifiedUsers();
    this.findTempUsers();
  }

  private getStructureNames(params): string {
    const structuresLink = params.data?.structuresLink;
    if (!structuresLink || structuresLink.length === 0) {
      return '';
    }
    const structureNames = structuresLink.map((structure) => (structure as Structure)?.structureName);
    return structureNames.join('\n');
  }

  public onDeleteButtonClick(arg): void {
    this.deleteUser(arg.data, false);
  }

  public onJobChange(arg): void {
    const selectedJob = this.validatedJobs.find((job) => job.name === arg.newValue);
    if (selectedJob === undefined && arg.oldValue !== undefined) {
      this.toggleEditJobModal(null, arg.oldValue, arg);
      return;
    }
    if (selectedJob && arg.oldValue === undefined) {
      // user had no job assigned, and admin selected a job -> call setUserJob without modal
      this.adminService.setUserJob(selectedJob._id, arg.data.id).subscribe(() => {
        arg.node.data[arg.colDef.field] = selectedJob;
        arg.api.refreshCells({ rowNodes: [arg.node], columns: [arg.column.colId] });

        this.notificationService.showSuccess('La fonction a bien été attribuée.');
      });
      return;
    }
    if (selectedJob === arg.oldValue || selectedJob.name === arg.oldValue.name) {
      arg.node.data[arg.colDef.field] = selectedJob;
      arg.api.refreshCells({ rowNodes: [arg.node], columns: [arg.column.colId] });
      return;
    }
    this.toggleEditJobModal(selectedJob, arg.oldValue, arg);
  }

  public onEmployerChange(arg): void {
    const selectedEmployer = this.validatedEmployers.find((employ) => employ.name === arg.newValue);
    if (selectedEmployer === undefined && arg.oldValue !== undefined) {
      this.toggleEditEmployerModal(null, arg.oldValue, arg);
      return;
    }
    if (selectedEmployer && arg.oldValue === undefined) {
      // user had no employer assigned, and admin selected a employer -> call setUserEmployer
      this.adminService.setUserEmployer(selectedEmployer._id, arg.data.id).subscribe(() => {
        arg.node.data[arg.colDef.field] = selectedEmployer;
        arg.api.refreshCells({ rowNodes: [arg.node], columns: [arg.column.colId] });
        this.notificationService.showSuccess("L'employeur a bien été attribué.");
      });
      return;
    }
    if (selectedEmployer === arg.oldValue || selectedEmployer.name === arg.oldValue.name) {
      arg.node.data[arg.colDef.field] = selectedEmployer;
      arg.api.refreshCells({ rowNodes: [arg.node], columns: [arg.column.colId] });
      return;
    }
    this.toggleEditEmployerModal(selectedEmployer, arg.oldValue, arg);
  }

  public deleteUser(user: User, shouldDelete: boolean): void {
    this.toggleDeleteModal(user);
    if (shouldDelete) {
      this.adminService.deleteUser(user._id).subscribe((data) => {
        this.unAttachedUsers = this.unAttachedUsers.filter((obj) => obj._id !== data._id);
        this.unVerifiedUsers = this.unVerifiedUsers.filter((obj) => obj._id !== data._id);
        this.attachedUsers = this.attachedUsers.filter((obj) => obj._id !== data._id);
      });
    }
  }

  public editJob(newJob: Job, oldJob: Job, shouldEdit: boolean, context?): void {
    this.toggleEditJobModal(newJob, oldJob, context);
    if (shouldEdit) {
      if (newJob) {
        // store newJob otherwise cell will only keep name value
        this.adminService.setUserJob(newJob._id, context.node.data._id).subscribe((data) => {
          context.node.data[context.colDef.field] = data.job;
          context.api.refreshCells({ rowNodes: [context.node], columns: [context.column.colId] });
          this.notificationService.showSuccess("L'opération a réussie.");
        });
      } else {
        this.adminService.removeUserJob(context.node.data._id).subscribe((data) => {
          context.node.data[context.colDef.field] = data.job;
          context.api.refreshCells({ rowNodes: [context.node], columns: [context.column.colId] });
          this.notificationService.showSuccess("L'opération a réussie.");
        });
      }
    } else {
      if (context.oldValue?.name) {
        context.node.data[context.colDef.field] = context.oldValue;
        context.api.refreshCells({ rowNodes: [context.node], columns: [context.column.colId] });
      }
    }
  }

  public editEmployer(newEmployer: Employer, oldEmployer: Employer, shouldEdit: boolean, context?): void {
    this.toggleEditEmployerModal(newEmployer, oldEmployer, context);
    if (shouldEdit) {
      if (newEmployer) {
        this.adminService.setUserEmployer(newEmployer._id, context.node.data._id).subscribe((data) => {
          context.node.data[context.colDef.field] = data.employer;
          context.api.refreshCells({ rowNodes: [context.node], columns: [context.column.colId] });
          this.notificationService.showSuccess("L'opération a réussie.");
        });
      } else {
        this.adminService.removeUserEmployer(context.node.data._id).subscribe((data) => {
          context.node.data[context.colDef.field] = data.employer;
          context.api.refreshCells({ rowNodes: [context.node], columns: [context.column.colId] });
          this.notificationService.showSuccess("L'opération a réussie.");
        });
      }
    } else {
      if (context.oldValue?.name) {
        context.node.data[context.colDef.field] = context.oldValue;
        context.api.refreshCells({ rowNodes: [context.node], columns: [context.column.colId] });
      }
    }
  }

  public toggleDeleteModal(userToDelete: User): void {
    this.userToDelete = userToDelete;
    this.deleteModalOpened = !this.deleteModalOpened;
  }

  public toggleEditJobModal(newValue: Job, oldValue: Job, dataGrid): void {
    this.changingJobs = [newValue, oldValue];
    this.contextRow = dataGrid;
    this.editJobModalOpened = !this.editJobModalOpened;
  }

  public toggleEditEmployerModal(newValue: Employer, oldValue: Employer, dataGrid): void {
    this.changingEmployers = [newValue, oldValue];
    this.contextRow = dataGrid;
    this.editEmployerModalOpened = !this.editEmployerModalOpened;
  }

  public findValidatedJobs(): void {
    this.adminService.getJobs().subscribe((jobs) => {
      this.validatedJobs = jobs;
      this.validatedJobsName.push('');
      jobs.map((job) => {
        this.validatedJobsName.push(job.name);
      });
    });
  }

  public findValidatedEmployers(): void {
    this.adminService.getEmployers().subscribe((employers) => {
      this.validatedEmployers = employers;
      this.validatedEmployersName.push('');
      employers.map((employer) => {
        this.validatedEmployersName.push(employer.name);
      });
    });
  }

  public findAttachedUsers(): void {
    this.adminService.getAttachedUsers().subscribe((users) => {
      this.attachedUsers = users;
      this.attachedUsers.forEach((user) => {
        user._id = user.id;
      });
    });
  }

  public findUnAttachedUsers(): void {
    this.adminService.getUnAttachedUsers().subscribe((users) => {
      this.unAttachedUsers = users;
      this.unAttachedUsers.forEach((user) => {
        user._id = user.id;
      });
    });
  }

  public findUnVerifiedUsers(): void {
    this.adminService.getUnVerifiedUsers().subscribe((users) => {
      this.unVerifiedUsers = users;
      this.unVerifiedUsers.forEach((user) => {
        user._id = user.id;
      });
    });
  }

  public findTempUsers(): void {
    this.tempUserService.getAllTempUsers().subscribe((data) => {
      this.tempUsers = data;
    });
  }

  public getRowHeight(params): number {
    const structures = params.data.structures || [];
    return structures.length !== 0 ? structures.length * 40 : 40;
  }

  public onGridReady(params: GridReadyEvent, status: UsersStatus): void {
    if (status === 'attached') {
      this.gridApiAttached = params.api;
    } else if (status === 'unattached') {
      this.gridApiUnattached = params.api;
    }
  }

  public exportUsers(status: UsersStatus): void {
    const gridApi = status === 'attached' ? this.gridApiAttached : this.gridApiUnattached;

    if (!gridApi) {
      throw new Error('Grid API not found for the selected status.');
    }

    const exportParams: CsvExportParams = {
      columnKeys: ['name', 'surname', 'email', 'job', 'structures'],
      processCellCallback: (params) => this.formatCells(params),
      fileName: `RESIN - export utilisateurs ${status === 'attached' ? 'rattachés' : 'non rattachés'}`,
    };

    gridApi.exportDataAsCsv(exportParams);
  }

  private formatCells(params: ProcessCellForExportParams): string {
    const cellRenderer = params.column.getColDef().cellRenderer;

    if (!cellRenderer) {
      return this.formatRegularCell(params);
    }

    switch (cellRenderer) {
      case 'jobRenderer':
        return this.formatJobCell(params.value);
      case 'administredStructuresComponent':
        return this.formatStructureCell(params.value);
      default:
        return params.value;
    }
  }

  private formatRegularCell(params: any): string {
    return params?.value;
  }

  private formatJobCell(params: any): string {
    return params?.name || '';
  }

  private formatStructureCell(params: any[]): string {
    if (params?.length > 0) {
      const structureNames = params.map((structure: Structure) => structure.structureName);
      return structureNames.join(', ');
    }
    return '';
  }
}
