import { Component, EventEmitter, Input, OnChanges, Output, SimpleChanges } from '@angular/core';
import { UntypedFormGroup } from '@angular/forms';
import { Router } from '@angular/router';
import { firstValueFrom } from 'rxjs';
import { User } from '../../models/user.model';
import { AuthService } from '../../services/auth.service';
import { NewsletterService } from '../../services/newsletter.service';
import { NotificationService } from '../../services/notification.service';
import { ButtonTypeV3 } from '../../shared/components/v3/button/button-type.enum';
import { Utils } from '../../utils/utils';
import { accountFormStep } from '../form-view/account-form/accountFormStep.enum';
import { formType } from '../form-view/formType.enum';
import { personalOfferFormStep } from '../form-view/personal-offer-form/personalOfferFormStep.enum';
import { profileFormStep } from '../form-view/profile-form/profileFormStep.enum';
import { structureFormStep } from '../form-view/structure-form/structureFormStep.enum';
import { stepType } from '../step.type';

@Component({
  selector: 'app-footer-form',
  templateUrl: './footer-form.component.html',
  styleUrls: ['./footer-form.component.scss'],
})
export class FooterFormComponent implements OnChanges {
  @Input() currentForm: formType;
  @Input() isValid: boolean;
  @Input() btnName: string[];
  @Input() nbPagesForm: number;
  @Input() form: UntypedFormGroup;
  @Input() linkedStructureId: string[] = null;
  @Input() acceptNewsletter: boolean;
  @Input() currentStep: stepType;
  @Input() hasOtherPersonalOffer: boolean;
  @Input() isPersonalOfferProfile: boolean;
  @Input() isEditMode: boolean;
  @Output() goNext = new EventEmitter<any>();
  @Output() goPrev = new EventEmitter<any>();
  @Output() endPage = new EventEmitter<any>();
  @Output() endForm = new EventEmitter<any>();
  @Output() changeCurrentStep = new EventEmitter<any>();
  @Output() saveEditedStructure = new EventEmitter<any>();

  public isLastFormStep = false;
  public isNextFormTransition = false;
  public buttonTypeEnumV3 = ButtonTypeV3;

  constructor(
    private authService: AuthService,
    public utils: Utils,
    private router: Router,
    private newsletterService: NewsletterService,
    private notificationService: NotificationService
  ) {}

  async ngOnChanges(changes: SimpleChanges): Promise<void> {
    if (changes.currentStep) {
      if (this.currentStep === accountFormStep.confirmEmailSentInfo && this.currentForm === formType.account) {
        this.isLastFormStep = true;
      }
      if (
        this.currentForm === formType.personaloffer &&
        this.currentStep === personalOfferFormStep.personalOfferFinishedInfo &&
        !this.hasOtherPersonalOffer
      ) {
        this.isNextFormTransition = true;
      }
      if (this.currentForm === formType.structure && this.currentStep === structureFormStep.noStructure) {
        this.isLastFormStep = true;
      }
    }
  }

  public goToNextPage(): void {
    this.goNext.emit();
  }

  public goToPreviousPage(): void {
    this.goPrev.emit();
  }

  public finishedModal(): void {
    this.endPage.emit();
  }

  public hasFinishButton(): boolean {
    return this.btnName.length === 3;
  }

  public goToHome(): void {
    this.router.navigateByUrl('news');
  }

  /**
   * Check if trainings are selected in order to ask for pricing
   */
  public isTrainingsSelected(): boolean {
    return this.form.value.categories.baseSkills.length > 0 || this.form.value.categories.advancedSkills.length > 0;
  }

  public prevPage(): void {
    if (this.isEditMode) {
      history.back();
    }

    if (this.currentForm === formType.structure) {
      if (this.currentStep === structureFormStep.structureType) {
        this.changeCurrentStep.emit(structureFormStep.structureFormTime);
        return;
      }
      if (this.currentStep === structureFormStep.structureWifi) {
        if (this.isTrainingsSelected()) {
          this.changeCurrentStep.emit(structureFormStep.structureTrainingPrice);
        } else {
          this.changeCurrentStep.emit(structureFormStep.structureTrainingType);
        }
        return;
      }
      if (this.currentStep === structureFormStep.structureTrainingType) {
        if (this.utils.isOtherAccompanimentSelected(this.form)) {
          this.changeCurrentStep.emit(structureFormStep.structureDigitalHelpingAccompanimentOther);
        } else {
          this.changeCurrentStep.emit(structureFormStep.structureDigitalHelpingAccompaniment);
        }
        return;
      }
    }

    this.goToPreviousPage();
  }

  public async nextPage(): Promise<void> {
    if (this.currentForm === formType.account && this.currentStep === accountFormStep.accountNewsletter) {
      const user = new User(this.form.value);
      // Create user with structure
      user.structuresLink = this.linkedStructureId;
      try {
        await firstValueFrom(this.authService.register(user));
        if (this.acceptNewsletter) {
          this.newsletterService.subscribe(user.email).subscribe();
        }
        document.getElementsByClassName('page')[0].scrollTo(0, 0);
        this.changeCurrentStep.emit(accountFormStep.confirmEmailSentInfo);
      } catch (error) {
        this.notificationService.showErrorPleaseRetry('Une erreur est survenue');
        return;
      }
      document.getElementsByClassName('page')[0].scrollTo(0, 0);
    }
    if (this.isProfileLastPage()) {
      this.endForm.emit({ formType: this.currentForm });
      return;
    }
    if (this.currentForm === formType.structure) {
      if (this.currentStep === structureFormStep.structureAccompanimentChoice) {
        const hasPlaceOfReception = this.form.get('placeOfReception').value;
        if (!hasPlaceOfReception) {
          this.changeCurrentStep.emit(structureFormStep.noStructure);
          return;
        }
      }
      if (this.currentStep === structureFormStep.structureFormTime) {
        this.changeCurrentStep.emit(structureFormStep.structureType);
        return;
      }
      // Check if trainings are selected in order to ask for pricing
      if (this.currentStep === structureFormStep.structureTrainingType) {
        if (this.isTrainingsSelected()) {
          this.changeCurrentStep.emit(structureFormStep.structureTrainingPrice);
        } else {
          this.changeCurrentStep.emit(structureFormStep.structureWifi);
        }
        return;
      }
      // Check if trainings are selected in order to ask for pricing
      if (this.currentStep === structureFormStep.structureDigitalHelpingAccompaniment) {
        if (this.utils.isOtherAccompanimentSelected(this.form)) {
          this.changeCurrentStep.emit(structureFormStep.structureDigitalHelpingAccompanimentOther);
        } else {
          this.changeCurrentStep.emit(structureFormStep.structureTrainingType);
        }
        return;
      }
      if (this.currentStep === structureFormStep.structureContactCompletion) {
        // TODO Go to send mail page and send the mail
        return;
      }
    }
    if (this.isPersonalOfferPage()) {
      this.endForm.emit({ formType: this.currentForm });
      return;
    }
    if (this.isStructureChoiceValid()) {
      this.endForm.emit({ formType: this.currentForm, formStep: this.currentStep });
      return;
    }
    if (this.isStructureLastPage()) {
      this.endForm.emit({ formType: this.currentForm, formStep: this.currentStep });
      return;
    }
    this.goToNextPage();
  }
  public saveEdit(): void {
    this.saveEditedStructure.emit();
  }

  private isStructureChoiceValid(): boolean {
    return (
      this.currentForm === formType.structure &&
      this.currentStep === structureFormStep.structureChoice &&
      this.form.value._id
    );
  }

  public isStructureLastPage(): boolean {
    return (
      this.currentForm === formType.structure &&
      (this.currentStep === structureFormStep.mailSentInfo ||
        this.currentStep === structureFormStep.structureCreationFinishedInfo ||
        this.currentStep === structureFormStep.noStructure)
    );
  }

  private isProfileLastPage(): boolean {
    return this.currentForm === formType.profile && this.currentStep === profileFormStep.profileJobSelection;
  }

  private isPersonalOfferPage(): boolean {
    // If the user is creating structure personalOffer from his profile, we don't ask if he has another structure and the last step is before the structure choice step
    const lastStep = this.isPersonalOfferProfile
      ? personalOfferFormStep.personalOfferStructureChoice - 1
      : personalOfferFormStep.personalOfferStructureChoice;
    return this.currentForm === formType.personaloffer && this.currentStep === lastStep;
  }
  public isPersonalOfferFirstPage(): boolean {
    return this.currentStep === personalOfferFormStep.personalOfferAccompaniment;
  }
}
