import { Component, EventEmitter, HostListener, Input, OnChanges, Output, SimpleChanges } from '@angular/core';
import L, { Map, MapOptions, geoJSON, latLng, layerGroup, tileLayer } from 'leaflet';
import * as _ from 'lodash';
import metropole from '../../../assets/geojson/metropole.json';
import { Structure } from '../../models/structure.model';
import { GeojsonService } from '../../services/geojson.service';
import { GeoJsonProperties } from '../models/geoJsonProperties.model';
import { MapService } from '../services/map.service';
import { MarkerType } from './markerType.enum';
import { ZoomLevel } from './zoomLevel.enum';

@Component({
  selector: 'app-map',
  templateUrl: './map.component.html',
  styleUrls: ['./map.component.scss'],
})
export class MapComponent implements OnChanges {
  @Input() public isOrientationForm = false;
  @Input() public isOrientationRdv = false;
  @Input() public structures: Structure[] = [];
  @Input() public structuresToPrint: Structure[] = [];
  @Input() public togglePopupId: string;
  @Input() public selectedMarkerId: string;
  @Input() public isMapPhone: boolean;
  @Input() public searchedValue: string | [number, number];
  @Output() public selectedStructure: EventEmitter<Structure> = new EventEmitter<Structure>();
  @Output() public clickOrientationButton: EventEmitter<Structure> = new EventEmitter<Structure>();
  private currentStructure: Structure;

  public map: Map;
  public mapOptions: MapOptions;
  public zoomOptions = { animate: true, duration: 0.5 };

  // Add listener on the popup button to show details of structure
  @HostListener('document:click', ['$event'])
  public clickout(event): void {
    if (event.target.classList.contains('btnShowDetails')) {
      this.selectedStructure.emit(this.currentStructure);
    }
    if (event.target.classList.contains('add')) {
      this.clickOrientationButton.emit(this.currentStructure);
      this.getStructuresPositions(this.structures);
    }
  }

  constructor(private mapService: MapService, private geoJsonService: GeojsonService) {
    this.initializeMapOptions();
  }

  ngOnChanges(changes: SimpleChanges): void {
    if (changes.searchedValue && !changes.searchedValue.firstChange) {
      const searchedValue = changes.searchedValue.currentValue;

      if (searchedValue) {
        this.processTownCoordinate(searchedValue);
      } else {
        this.resetMap();
      }
    }

    if (changes.isMapPhone && this.isMapPhone) {
      setTimeout(() => {
        this.map.invalidateSize();
      }, 0);
    }

    if (changes.structures) {
      const previousStructures = changes.structures.previousValue;

      this.handleStructurePosition(previousStructures);
    }

    this.handleMapMarkerTooltip(changes);
    this.handleSelectedMapMarker(changes);
    this.handlePrintableMarkers();

    this.closePreviousMarker(changes);
  }

  private resetMap(): void {
    this.map.setView(this.mapOptions.center, this.mapOptions.zoom);
  }

  private handleMapMarkerTooltip(changes: SimpleChanges): void {
    const previousId = changes.togglePopupId?.previousValue;
    const currentId = changes.togglePopupId?.currentValue;

    if (previousId !== undefined) {
      const isToPrint = this.isToPrint(previousId);

      if (isToPrint) {
        this.mapService.setAddedToListMarker(previousId, this.getMarkerTypeByStructureId(previousId));
      } else {
        this.mapService.setInactiveMarker(previousId, this.getMarkerTypeByStructureId(previousId));
      }
    }

    if (currentId !== undefined) {
      this.mapService.setActiveMarker(currentId, this.getMarkerTypeByStructureId(currentId));
    }
  }

  private handleSelectedMapMarker(changes: SimpleChanges): void {
    const selectedMarkerId = this.selectedMarkerId;

    if (changes.selectedMarkerId && this.map) {
      const previousId = changes.selectedMarkerId.previousValue;

      this.map.closePopup();

      if (selectedMarkerId === undefined) {
        this.mapService.setDefaultMarker(previousId, this.getMarkerTypeByStructureId(previousId));
        this.resetMap();
      }
    }

    if (this.mapService.getMarker(selectedMarkerId)) {
      this.mapService.setSelectedMarker(selectedMarkerId, this.getMarkerTypeByStructureId(selectedMarkerId));
      this.centerLeafletMapOnMarker(selectedMarkerId);
    }
  }

  private handlePrintableMarkers(): void {
    const structuresToPrint = this.structuresToPrint;

    if (structuresToPrint) {
      structuresToPrint.forEach((structure: Structure) => {
        this.mapService.setAddedToListMarker(structure._id, this.getMarkerTypeByStructureId(structure._id));
      });
    }
  }

  public processTownCoordinate(queryString: string): void {
    this.geoJsonService.getTownshipCoord(queryString).subscribe(
      (townData) => {
        if (townData.length > 0) {
          const bounds = new L.LatLngBounds(townData.map((dataArray) => dataArray.reverse()));
          this.map.fitBounds(bounds);
        }
      },
      () => {
        this.map.flyTo(this.mapOptions.center, this.mapOptions.zoom, this.zoomOptions);
      }
    );
  }

  /**
   * Create a user position marker and center the map on it with a zoom level defined in ZoomLevel
   */
  public centerOnCoordinates(coords: [number, number]): void {
    this.mapService.createMarker(coords[1], coords[0], MarkerType.user, 'userLocation').addTo(this.map);
    this.map.flyTo(new L.LatLng(coords[1], coords[0]), ZoomLevel.userPosition, this.zoomOptions);
  }

  /**
   * Get structures positions and add marker corresponding to those positons on the map
   */
  private handleStructurePosition(previousStructuresValue: Structure[]): void {
    // If there is more structure than before, append them
    if (
      previousStructuresValue &&
      previousStructuresValue.length > 0 &&
      previousStructuresValue.length < this.structures.length
    ) {
      this.getStructuresPositions(_.differenceWith(this.structures, previousStructuresValue, _.isEqual));
    } else if (this.structures) {
      this.map = this.mapService.cleanMap(this.map);
      this.getStructuresPositions(this.structures);
      this.structuresToPrint?.forEach((structure: Structure) => {
        this.mapService.setAddedToListMarker(structure._id, this.getMarkerTypeByStructureId(structure._id));
      });
    }
  }

  private isToPrint(id: string): boolean {
    return this.structuresToPrint.some((elem) => elem._id === id);
  }

  /**
   * Returns according marker type base on {MarkerType}
   */
  private getMarkerType(structure: Structure): MarkerType {
    return structure?.categories?.labelsQualifications?.includes('conseillerNumFranceServices')
      ? MarkerType.conseillerFrance
      : MarkerType.structure;
  }

  /**
   * Return the map marker type given a structure id
   */
  private getMarkerTypeByStructureId(id: string): MarkerType {
    return this.getMarkerType(this.structures.find((structure) => structure._id === id));
  }

  private getStructuresPositions(structureList: Structure[]): void {
    for (const structure of structureList) {
      if (!(structure.getLat() && structure.getLon())) {
        console.log(`Pas de position pour ${structure.structureName}`);
        continue;
      }
      this.mapService
        .createMarker(
          structure.getLat(),
          structure.getLon(),
          this.getMarkerType(structure),
          structure._id,
          this.buildPopup(structure)
        )
        .addTo(this.map)
        // store structure before user click on button
        .on('popupopen', () => {
          this.currentStructure = structure;
        });
    }
  }

  private buildPopup(structure: Structure): string {
    return (
      '<h1>' +
      structure.structureName +
      '</h1>' +
      '<p>' +
      (structure.structureType?.name || '') +
      '</p>' +
      '<button type="button" class="btnShowDetails">Voir la structure</button>'
    );
  }

  private buildMdmPopUp(mdmProperties: GeoJsonProperties): string {
    return `<h1>${mdmProperties.nom}</h1><p>${mdmProperties.adresse}</p>`;
  }

  /**
   * Add marker when map is ready to be shown
   */
  public onMapReady(map: Map): void {
    this.map = map;
    if (this.searchedValue) {
      if (Array.isArray(this.searchedValue)) {
        this.centerOnCoordinates(this.searchedValue);
      }
    }
  }

  /**
   * Init map options :
   * - Metropole bounds based on a WMS service hosted by data.grandlyon.com
   * - Map Layer based on open street maps
   */
  private initializeMapOptions(): void {
    // Init mdm
    this.initMDMLayer();
    // Init WMS service with param from data.grandlyon.com
    layerGroup();
    const carteLayer = tileLayer('https://{s}.basemaps.cartocdn.com/rastertiles/voyager/{z}/{x}/{y}{r}.png', {
      attribution:
        '&copy; <a href="http://www.openstreetmap.org/copyright">OpenStreetMap</a>, &copy; <a href="https://carto.com/attributions">CARTO</a>',
      maxZoom: ZoomLevel.max,
    });
    // Center is set on townhall
    // Zoom is blocked on 11 to prevent people to zoom out from metropole
    this.mapOptions = {
      center: latLng(45.764043, 4.835659),
      maxZoom: ZoomLevel.max,
      zoom: ZoomLevel.regular,
      minZoom: ZoomLevel.min,
      layers: [carteLayer],
    };
  }

  private initMDMLayer(): void {
    this.geoJsonService.getMDMGeoJson().subscribe((res) => {
      res.forEach((mdm) => {
        this.mapService
          .createMarker(
            mdm.geometry.getLat(),
            mdm.geometry.getLon(),
            MarkerType.mdm,
            null,
            this.buildMdmPopUp(mdm.properties)
          )
          .addTo(this.map);
      });
      this.initMetropoleLayer();
    });
  }

  private centerLeafletMapOnMarker(markerId: string): void {
    if (this.mapService.getMarker(markerId)) {
      const marker = this.mapService.getMarker(markerId);
      const latLngs = marker.getLatLng();
      this.map.flyTo(new L.LatLng(latLngs.lat, latLngs.lng), ZoomLevel.max, this.zoomOptions);
    }
  }

  private initMetropoleLayer(): void {
    this.map.addLayer(
      geoJSON(
        {
          type: metropole.features[0].geometry.type,
          coordinates: metropole.features[0].geometry.coordinates,
        } as any,
        { style: () => ({ color: '#a00000', fillOpacity: 0, weight: 1 }) }
      )
    );
  }

  /**
   * Close previous markers
   * - if structure is closed
   * - if a new marker is selected
   */
  private closePreviousMarker(changes: SimpleChanges): void {
    if (
      (changes.selectedMarkerId?.currentValue === undefined && changes.selectedMarkerId?.previousValue) ||
      changes.selectedMarkerId?.currentValue !== changes.selectedMarkerId?.previousValue
    ) {
      this.mapService.setInactiveMarker(
        changes.selectedMarkerId.previousValue,
        this.getMarkerTypeByStructureId(changes.selectedMarkerId.previousValue)
      );
    }
  }
}
