import { Injectable } from '@angular/core';
import { DivIcon, Map, Marker } from 'leaflet';
import { Layers } from '../components/layers.enum';
import { MarkerType } from '../components/markerType.enum';
import {
  markerIcon,
  markerIconActive,
  markerIconAddedToList,
  markerIconHover,
  markerIconMdm,
  markerIconMdmActive,
  userLocationIcon,
} from './marker';
@Injectable({
  providedIn: 'root',
})
export class MapService {
  private static markersList = {};

  public createMarker(lat: number, lon: number, markerType: MarkerType, id?: string, popup?: string): Marker {
    const marker = new Marker([lat, lon], {
      icon: this.getMarkerIcon(markerType),
      attribution: this.getLayerAttribution(markerType),
    });

    if (popup) {
      marker.bindPopup(popup);
    }

    if (id) {
      MapService.markersList[id] = marker;
    }
    return marker;
  }

  private getLayerAttribution(markerType: MarkerType): string {
    if (markerType === MarkerType.mdm) {
      return Layers.mdm;
    } else if (markerType === MarkerType.user) {
      return Layers.user;
    } else {
      return Layers.structure;
    }
  }

  // Note: Mark IconFranceService has been removed temporarily on order to rework on business needs.
  // This comment is applied for the next 4 methods
  private getMarkerIcon(markerType: MarkerType): DivIcon {
    switch (markerType) {
      case MarkerType.mdm:
        return markerIconMdm;
      case MarkerType.conseillerFrance:
        return markerIcon;
      case MarkerType.user:
        return userLocationIcon;
      default:
        return markerIcon;
    }
  }

  private getActiveMarkerIcon(markerType: MarkerType): DivIcon {
    switch (markerType) {
      case MarkerType.mdm:
        return markerIconMdmActive;
      case MarkerType.conseillerFrance:
        // return markerIconFranceServiceActive;
        return markerIconActive;
      case MarkerType.user:
        return userLocationIcon;
      default:
        return markerIconActive;
    }
  }

  private getAddedToListMarkerIcon(markerType: MarkerType): DivIcon {
    switch (markerType) {
      case MarkerType.conseillerFrance:
        // return markerIconFranceServiceAddedToList;
        return markerIconAddedToList;
      case MarkerType.user:
        return userLocationIcon;
      default:
        return markerIconAddedToList;
    }
  }

  private getHoverMarkerIcon(markerType: MarkerType): DivIcon {
    switch (markerType) {
      case MarkerType.conseillerFrance:
        return markerIconHover;
      case MarkerType.user:
        return userLocationIcon;
      default:
        return markerIconHover;
    }
  }

  /**
   * @param id marker id
   */
  public setActiveMarker(id: string, type: MarkerType = MarkerType.structure): void {
    this.getMarker(id)?.setIcon(this.getHoverMarkerIcon(type));
  }

  public setAddedToListMarker(id: string, type: MarkerType = MarkerType.structure): void {
    this.getMarker(id)?.setIcon(this.getAddedToListMarkerIcon(type));
  }

  public setInactiveMarker(id: string, type: MarkerType = MarkerType.structure): void {
    // To skip mouseleave when user emit click on structure list
    this.getMarker(id)?.setIcon(this.getMarkerIcon(type));
  }

  /**
   * Set a marker as selected by changing icon color
   * @param id markerId
   * @param html html to display
   */
  public setSelectedMarker(id: string, type: MarkerType = MarkerType.structure): void {
    if (id) {
      this.getMarker(id)?.setIcon(this.getActiveMarkerIcon(type));
    }
  }

  /**
   * Set a marker as selected by changing icon color
   * @param id markerId
   * @param html html to display
   */
  public setDefaultMarker(id: string, type: MarkerType = MarkerType.structure): void {
    if (id) {
      this.getMarker(id).setIcon(this.getMarkerIcon(type));
    }
  }

  /**
   * Get marker by id
   */
  public getMarker(id: string): Marker {
    return MapService.markersList[id] ? MapService.markersList[id] : null;
  }

  public cleanMap(map: Map): Map {
    MapService.markersList = {};
    if (map) {
      map.eachLayer((layer) => {
        if (layer instanceof Marker && layer.options.attribution === Layers.structure) {
          map.removeLayer(layer);
        }
      });
    }
    return map;
  }
}
