import { Component, OnInit } from '@angular/core';
import { AbstractControl, UntypedFormBuilder, UntypedFormGroup, Validators } from '@angular/forms';
import { Router } from '@angular/router';
import { lastValueFrom } from 'rxjs';
import { NewsletterService } from '../services/newsletter.service';
import { NotificationService } from '../services/notification.service';
import { ButtonTypeV3 } from '../shared/components/v3/button/button-type.enum';
import { CustomRegExp } from '../utils/CustomRegExp';

@Component({
  selector: 'app-newsletter-subscription',
  templateUrl: './newsletter-subscription.component.html',
  styleUrls: ['./newsletter-subscription.component.scss'],
})
export class NewsletterSubscriptionComponent implements OnInit {
  public subscriptionForm: UntypedFormGroup;
  public loading = false;
  public submitted = false;
  public subscriptionMod: boolean;
  public buttonTypeEnumV3 = ButtonTypeV3;

  constructor(
    private formBuilder: UntypedFormBuilder,
    private newsletterService: NewsletterService,
    private router: Router,
    private notificationService: NotificationService
  ) {}

  ngOnInit(): void {
    this.subscriptionForm = this.formBuilder.group({
      email: ['', [Validators.required, Validators.pattern(CustomRegExp.EMAIL)]],
    });
    if (this.router.url === '/newsletter') {
      this.subscriptionMod = true;
    }
    if (this.router.url === '/newsletter-unsubscription') {
      this.subscriptionMod = false;
    }
  }

  get f(): Record<string, AbstractControl> {
    return this.subscriptionForm.controls;
  }

  public async onSubmit(): Promise<void> {
    this.submitted = true;

    if (this.subscriptionForm.invalid) {
      return;
    }
    this.loading = true;
    if (this.subscriptionMod) {
      await lastValueFrom(this.newsletterService.subscribe(this.f.email.value))
        .then(() => {
          this.notificationService.showSuccess(`L'email ${this.f.email.value} a bien été ajouté à la newsletter`);
          this.router.navigate(['']);
        })
        .catch((error) => {
          this.loading = false;
          if (error.status === 418) {
            this.notificationService.showError(
              "Merci de vérifier l'email que vous souhaitez inscrire",
              `L'email ${this.f.email.value} semble factice ou invalide`
            );
          } else {
            this.notificationService.showErrorPleaseRetry(
              `L'email ${this.f.email.value} n'a pas pu être ajouté à la newsletter`
            );
          }
        });
    }
    if (!this.subscriptionMod) {
      await lastValueFrom(this.newsletterService.unsubscribe(this.f.email.value))
        .then(() => {
          this.notificationService.showSuccess(`L'email ${this.f.email.value} a bien été retiré de la newsletter`);
          this.router.navigate(['']);
        })
        .catch((error) => {
          this.loading = false;
          if (error.status === 404) {
            this.notificationService.showError(
              "Merci de vérifier l'email que vous souhaitez désinscrire",
              `L'email ${this.f.email.value} n'est pas inscrit à la newsletter`
            );
          } else {
            this.notificationService.showErrorPleaseRetry(
              `L'email ${this.f.email.value} n'a pas pu être retiré de la newsletter`
            );
          }
        });
    }
  }
}
