import { UntypedFormArray, UntypedFormControl, UntypedFormGroup, Validators } from '@angular/forms';
import { structureFormStep } from '../form/form-view/structure-form/structureFormStep.enum';
import { Day } from '../models/day.model';
import { Structure } from '../models/structure.model';
import { Time } from '../models/time.model';
import { Week } from '../models/week.model';
import { CustomRegExp } from './CustomRegExp';
import { FormValidators } from './formValidators';

export interface IStructureSummary {
  step: structureFormStep;
  name: string;
  inputs: string[] | null;
  hideForEdit?: boolean;
}
export class formUtils {
  public structureSummary: IStructureSummary[] = [
    { step: structureFormStep.structureNameAndAddress, name: 'Nom et adresse', inputs: ['structureName', 'address'] },
    { step: structureFormStep.structureContact, name: 'Téléphone', inputs: ['contactMail', 'contactPhone'] },
    { step: structureFormStep.structureType, name: 'Type de structure', inputs: ['structureType'] },
    { step: structureFormStep.structureAccessModality, name: "Modalités d'accueil", inputs: ['accessModality'] },
    { step: structureFormStep.structureHours, name: "Horaires d'ouverture", inputs: ['exceptionalClosures'] },
    {
      step: structureFormStep.structurePmr,
      name: 'Accessibilité pour les personnes à mobilité réduite',
      inputs: ['pmrAccess'],
    },
    {
      step: structureFormStep.structureWebAndSocialNetwork,
      name: 'Présence sur internet',
      inputs: ['website', 'facebook', 'twitter', 'instagram'],
    },
    { step: structureFormStep.structurePublicTarget, name: 'Public admis', inputs: ['publics'] },
    { step: structureFormStep.structurePublicTarget, name: 'Public admis - Optionnel', inputs: ['publics'] },
    {
      step: structureFormStep.structureDigitalHelpingAccompaniment,
      name: 'Aides au numérique',
      inputs: ['onlineProcedures'],
    },
    { step: structureFormStep.structureTrainingType, name: 'Ateliers au numérique proposés', inputs: null },
    { step: structureFormStep.structureTrainingPrice, name: 'Gratuité des ateliers', inputs: ['freeWorkShop'] },
    { step: structureFormStep.structureWifi, name: 'Wifi', inputs: ['selfServiceMaterial'] },
    { step: structureFormStep.structureEquipments, name: 'Equipements mis à disposition', inputs: null },
    { step: structureFormStep.structureLabels, name: 'Labélisations proposées', inputs: null },
    { step: structureFormStep.structureDescription, name: 'Présentation de la structure', inputs: null },
    { step: structureFormStep.structureCovidInfo, name: 'Informations spécifiques à la période COVID', inputs: null },
    {
      step: structureFormStep.structureConsent,
      name: 'Partage de données sur data.grandlyon.com',
      inputs: ['dataShareConsentDate'],
    },
    {
      step: structureFormStep.structureContactCompletion,
      name: 'Personne contact informations',
      inputs: ['contactPersonFirstname', 'contactPersonLastname', 'contactPersonEmail'],
      hideForEdit: true,
    },
    {
      step: structureFormStep.structureAccompanimentChoice,
      name: "Lieu d'accueil",
      inputs: ['placeOfReception'],
      hideForEdit: true,
    },
  ];

  /**
   * @param isEditMode in edit mode, some validations are not required
   */
  public createStructureForm(structure: Structure, isEditMode?: boolean): UntypedFormGroup {
    return new UntypedFormGroup(
      {
        _id: new UntypedFormControl(structure._id, Validators.required),
        coord: new UntypedFormControl(structure.coord),
        structureType: new UntypedFormControl(structure.structureType?._id, Validators.required),
        structureName: new UntypedFormControl(structure.structureName, Validators.required),
        description: new UntypedFormControl(structure.description),
        lockdownActivity: new UntypedFormControl(structure.lockdownActivity),
        address: new UntypedFormGroup({
          numero: new UntypedFormControl(structure.address.numero),
          street: new UntypedFormControl(structure.address.street, Validators.required),
          commune: new UntypedFormControl(structure.address.commune, Validators.required),
        }),
        contactMail: new UntypedFormControl(structure.contactMail === '' ? null : structure.contactMail, [
          Validators.pattern(CustomRegExp.EMAIL),
        ]),
        contactPhone: new UntypedFormControl(structure.contactPhone, [Validators.pattern(CustomRegExp.PHONE)]),
        contactPersonFirstname: new UntypedFormControl(
          structure.contactPersonLastName,
          !isEditMode && Validators.required
        ),
        contactPersonLastname: new UntypedFormControl(
          structure.contactPersonLastName,
          !isEditMode && Validators.required
        ),
        contactPersonEmail: new UntypedFormControl(
          structure.contactPersonEmail,
          !isEditMode && [Validators.pattern(CustomRegExp.EMAIL), Validators.required]
        ),
        website: new UntypedFormControl(structure.website, Validators.pattern(CustomRegExp.WEBSITE)),
        facebook: new UntypedFormControl(structure.facebook, Validators.pattern(CustomRegExp.FACEBOOK)),
        twitter: new UntypedFormControl(structure.twitter, Validators.pattern(CustomRegExp.TWITTER)),
        instagram: new UntypedFormControl(structure.instagram, Validators.pattern(CustomRegExp.INSTAGRAM)),
        linkedin: new UntypedFormControl(structure.linkedin, Validators.pattern(CustomRegExp.LINKEDIN)),
        hours: new UntypedFormGroup({}),
        pmrAccess: new UntypedFormControl(structure.pmrAccess, Validators.required),
        placeOfReception: new UntypedFormControl(structure.placeOfReception, !isEditMode && Validators.required),
        choiceCompletion: new UntypedFormControl(structure.choiceCompletion, !isEditMode && Validators.required),
        exceptionalClosures: new UntypedFormControl(structure.exceptionalClosures),
        categories: new UntypedFormGroup({
          labelsQualifications: this.loadArrayForCheckbox(structure.categories?.labelsQualifications, false),
          accessModality: this.loadArrayForCheckbox(structure.categories?.accessModality, true),
          publicsAccompaniment: this.loadArrayForCheckbox(structure.categories?.age, false),
          onlineProcedures: this.loadArrayForCheckbox(structure.categories?.onlineProcedures, false),
          handicaps: this.loadArrayForCheckbox(structure.categories?.handicaps, false),
          age: this.loadArrayForCheckbox(structure.categories?.age, true),
          languageAndIlliteracy: this.loadArrayForCheckbox(structure.categories?.languageAndIlliteracy, false),
          selfServiceMaterial: this.loadArrayForCheckbox(structure.categories?.selfServiceMaterial, false),
          genre: this.loadArrayForCheckbox(structure.categories?.genre, false),
          baseSkills: new UntypedFormControl(structure.categories?.baseSkills),
          advancedSkills: new UntypedFormControl(structure.categories?.advancedSkills),
          solidarityMaterial: this.loadArrayForCheckbox(structure.categories?.solidarityMaterial, false),
        }),
        // TODO: remettre ou migrer les données de accompagnements à distance
        remoteAccompaniment: new UntypedFormControl(false),
        otherDescription: new UntypedFormControl(structure.otherDescription),
        nbComputers: new UntypedFormControl(
          structure.categories.selfServiceMaterial.includes('computer') ? structure.nbComputers : 0,
          [
            Validators.required,
            Validators.pattern(CustomRegExp.NO_NEGATIVE_NUMBER),
            Validators.min(0),
            Validators.max(1000),
          ]
        ),
        nbPrinters: new UntypedFormControl(
          structure.categories.selfServiceMaterial.includes('printer') ? structure.nbPrinters : 0,
          [
            Validators.required,
            Validators.pattern(CustomRegExp.NO_NEGATIVE_NUMBER),
            Validators.min(0),
            Validators.max(1000),
          ]
        ),
        nbScanners: new UntypedFormControl(
          structure.categories.selfServiceMaterial.includes('scanner') ? structure.nbScanners : 0,
          [
            Validators.required,
            Validators.pattern(CustomRegExp.NO_NEGATIVE_NUMBER),
            Validators.min(0),
            Validators.max(1000),
          ]
        ),
        freeWorkShop: new UntypedFormControl(structure.freeWorkShop, [Validators.required]),
        freeWifi: new UntypedFormControl(
          structure.categories.selfServiceMaterial.includes('wifiEnAccesLibre') ? true : null,
          isEditMode ? [] : [Validators.required]
        ), // Field used has a pivot, not sent in final form.
        dataShareConsentDate: new UntypedFormControl(structure.dataShareConsentDate),
        personalOffers: new UntypedFormControl(structure.personalOffers),
      },
      FormValidators.requirePhoneOrMail(['contactPhone', 'contactMail'])
    );
  }

  public loadArrayForCheckbox(array: string[], isRequired: boolean): UntypedFormArray {
    return new UntypedFormArray(
      array.map((str) => new UntypedFormControl(str)),
      isRequired ? Validators.required : Validators.nullValidator
    );
  }

  public createDay(day: Day): UntypedFormGroup {
    return new UntypedFormGroup({
      open: new UntypedFormControl(day.open, Validators.required),
      time: new UntypedFormArray(day.time.map((oneTime) => this.createTime(oneTime))) as UntypedFormArray,
    });
  }
  public createTime(time: Time): UntypedFormGroup {
    return new UntypedFormGroup({
      opening: new UntypedFormControl(time.opening),
      closing: new UntypedFormControl(time.closing),
    });
  }
  public createHoursForm(structure: Structure): UntypedFormGroup {
    if (!structure.hours) {
      structure.hours = new Week();
    }
    return new UntypedFormGroup({
      monday: this.createDay(structure.hours?.monday),
      tuesday: this.createDay(structure.hours?.tuesday),
      wednesday: this.createDay(structure.hours?.wednesday),
      thursday: this.createDay(structure.hours?.thursday),
      friday: this.createDay(structure.hours?.friday),
      saturday: this.createDay(structure.hours?.saturday),
      sunday: this.createDay(structure.hours?.sunday),
    });
  }

  public isInCategoryArray(term: string, formControlName: string, form: UntypedFormGroup): boolean {
    if (form.get('categories').get(formControlName).value) {
      return form.get('categories').get(formControlName).value.includes(term);
    }
    return false;
  }
}
